@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Component/Core.pm 31304 2007-11-29T11:56:44.884140Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Component/Core.pm 8909 2007-11-12T01:04:17.979695Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -6,11 +6,9 @@
 package Gungho::Component::Core;
 use strict;
 use warnings;
-use base qw(Gungho::Component);
 use Carp ();
 use Config::Any;
 use Class::Inspector;
-use Event::Notify;
 use UNIVERSAL::isa;
 use UNIVERSAL::require;
 use HTTP::Status qw(status_message);
@@ -20,13 +18,10 @@ use Gungho::Request;
 use Gungho::Response;
 use Gungho::Util;
 
-__PACKAGE__->mk_classdata('notify_hub');
-
 sub setup
 {
     my $c = shift;
 
-    $c->notify_hub( Event::Notify->new );
     $c->setup_log();
     $c->setup_provider();
     $c->setup_handler();
@@ -43,7 +38,7 @@ sub setup_log
     my $log_config = { %{$c->config->{log} || { logs => [] }} };
     my $module     = delete $log_config->{module} || 'Simple';
     my $pkg        = $c->load_gungho_module($module, 'Log');
-    my $log        = $pkg->new(config => $log_config->{config} || $log_config);
+    my $log        = $pkg->new(config => $log_config);
 
     $log->setup($c);
     $c->log($log);
@@ -69,7 +64,9 @@ sub setup_provider
                 callback => $code
             }
         }
-    } elsif ( $ref ne 'HASH') {
+    }
+
+    if ( $ref ne 'HASH') {
         Carp::croak("Gungho requires a provider");
     }
 
@@ -120,7 +117,9 @@ sub setup_handler
                 callback => $code
             }
         }
-    } elsif ( $ref ne 'HASH') {
+    }
+
+    if ( $ref ne 'HASH') {
         Carp::croak("Gungho requires a handler");
     }
 
@@ -143,6 +142,31 @@ sub setup_plugins
     }
 }
 
+sub register_hook
+{
+    my $c = shift;
+    my $hooks = $c->hooks;
+    while(@_) {
+        my($name, $hook) = splice(@_, 0, 2);
+        $hooks->{$name} ||= [];
+        push @{ $hooks->{$name} }, $hook;
+    }
+}
+
+sub run_hook
+{
+    my $c = shift;
+    my $name = shift;
+    my $hooks = $c->hooks->{$name} || [];
+    foreach my $hook (@{ $hooks }) {
+        if (ref($hook) eq 'CODE') {
+            $hook->($c, @_);
+        } else {
+            $hook->execute($c, @_);
+        }
+    }
+}
+
 sub has_feature
 {
     my ($c, $name) = @_;
@@ -165,7 +189,7 @@ sub dispatch_requests
     my $c = shift;
     if ($c->is_running) {
         $c->provider->dispatch($c, @_);
-        $c->notify('dispatch.dispatch_requests');
+        $c->run_hook('dispatch.dispatch_requests');
     }
 }
 
@@ -173,7 +197,7 @@ sub prepare_request
 {
     my $c = shift;
     my $req  = shift;
-    $c->notify('dispatch.prepare_request', $req);
+    $c->run_hook('dispatch.prepare_request', $req);
     return $req;
 }
 
@@ -199,7 +223,7 @@ sub send_request
     my $c = shift;
     my $request = shift;
     $request = $c->prepare_request($request);
-    return $c->engine->send_request($c, $request);
+    $c->engine->send_request($c, $request);
 }
 
 sub pushback_request
@@ -275,40 +299,6 @@ sub _http_error
     return $r;
 }
 
-sub register_event
-{
-    my $c = shift;
-    $c->notify_hub->register_event(@_);
-}
-*register_hook = \&register_event;
-
-sub unregister_event
-{
-    my $c = shift;
-    $c->notify_hub->unregister_event(@_);
-}
-
-sub notify
-{
-    my ($c, $event, @args) = @_;
-    $c->notify_hub->notify($event, $c, @args);
-}
-*run_hook = \&notify;
-
-sub shutdown
-{
-    my ($c, $reason) = @_;
-
-    $reason ||= 'unknown reason';
-    $c->log->notice("Gungho received a shutdown request!: '$reason'");
-    $c->is_running(0);
-
-    # Tell everybody to shutdown
-    $c->provider->stop($reason);
-    $c->handler->stop($reason);
-    $c->engine->stop($reason);
-}
-
 1;
 
 __END__
@@ -353,25 +343,11 @@ Sets up the various components.
 
 =head2 register_hook($hook_name => $coderef[, $hook_name => $coderef])
 
-Is deprecated. Use register_event instead.
+Registers a hook to be run under the specified $hook_name
 
-=head2 register_event($event, $observer)
+=head2 run_hook($hook_name)
 
-Registers an observer that gets notified when $event happens. The $observer
-argument can be either an object implementing notify(), or a subroutine
-reference.
-
-=head2 unregister_event($event, $observer)
-
-Unregisters an observer from the specified event
-
-=head2 run_hook($hook_name, @args)
-
-Is deprecated. Use notify() instead.
-
-=head2 notify($event, @args)
-
-Notifies observers of an event.
+Runs all the hooks under the hook $hook_name
 
 =head2 has_requests
 
@@ -424,10 +400,4 @@ performed, and the module name is used as-is.
 Returns true if the given request is allowed to be fetched (this has nothing
 to do with authentication and such, and is purely internal)
 
-=head2 shutdown($reason)
-
-Shuts down Gungho. Call this if you want to tell the entire system to stop.
-This method in turn calls stop methods on the Engine, Provider, and Handler
-objects
-
 =cut
