@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Component/RobotRules.pm 3539 2007-10-17T15:48:38.501780Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Component/RobotRules.pm 8891 2007-11-10T14:10:30.426265Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 
@@ -30,15 +30,12 @@ sub send_request
     ;
     if ($allowed == -2) {
         $c->log->debug("Fetch for /robots.txt already scheduled for " . $request->original_uri->host_port);
-        Gungho::Exception::SendRequest::Handled->throw;
     } elsif ($allowed == -1) {
         $c->log->debug("No robot rules found for " . $request->original_uri->host_port . ", going to fetch one");
-        Gungho::Exception::SendRequest::Handled->throw;
     } elsif ($allowed) {
-        $c->maybe::next::method($request);
+        $c->next::method($request);
     } else {
         $c->log->debug($request->uri . " is disallowed by robot rules");
-        Gungho::Exception::SendRequest::Handled->throw;
     }
 }
 
@@ -58,7 +55,7 @@ sub allowed
         $uri->fragment(undef);
         my $req = Gungho::Request->new(GET => $uri);
         $req->notes('auto_robot_rules' => 1);
-        $c->provider->pushback_request( $c, $req );
+        $c->pushback_request( $req );
         return -1;
     } else {
         return $rule->allowed( $c, $request->uri );
@@ -74,10 +71,10 @@ sub handle_response
         $c->log->debug("Handling robots.txt response for " . $request->uri);
         $c->parse_robot_rules($request, $response);
         $c->dispatch_pending_robots_txt($request);
-        Gungho::Exception::HandleResponse::Handled->throw;
+        return;
     }
 
-    $c->maybe::next::method(@_);
+    $c->next::method(@_);
 }
 
 sub push_pending_robots_txt
@@ -92,7 +89,7 @@ sub dispatch_pending_robots_txt
 
     my $pending = $c->robot_rules_storage->get_pending_robots_txt($c, $request);
     if ($pending && ref $pending eq 'HASH') {
-        $c->provider->pushback_request( $c, $_ ) for values %$pending;
+        $c->pushback_request( $_ ) for values %$pending;
     }
 }
 
