@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Component/Throttle.pm 3234 2007-10-13T15:12:58.068532Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Component/Throttle.pm 8775 2007-11-08T09:53:44.496906Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 
@@ -8,7 +8,13 @@ use warnings;
 use base qw(Gungho::Component);
 
 sub feature_name { 'Throttle' }
-sub throttle { 1 }
+
+sub throttle
+{
+    my $c = shift;
+    $c->log->debug( $_[0]->url . " NOT throttled" );
+    return 1;
+}
 
 sub send_request
 {
@@ -17,9 +23,9 @@ sub send_request
     if (! $request->notes('original_host') && ! $c->throttle($request)) {
         $c->log->debug("Request " . $request->url . " (" . $request->id . ") was throttled");
         $c->provider->pushback_request($c, $request);
-        Gungho::Exception::SendRequest::Handled->throw();
+    } else {
+        $c->next::method($request);
     }
-    $c->maybe::next::method($request);
 }
 
 1;
@@ -88,6 +94,11 @@ the throttler argument in the configuration:
       max_items: 100
       interval: 3600
 
+Starting from 0.09003, you can stack throttlers. For example, you can throttle
+by Throttle::Simple first, and if Throttle::Simple allowed the request to
+go, then you can  throttle with Throttle::Domain as well to make sure that
+the same host doesn't get beaten up.
+
 =head1 METHODS
 
 =head2 feature_name
